/******************************************************************************************************************************
VERWENDUNG:	Prfgert
AUTHOR: 	  Marco Glietsch / PHILIPP GAREIS
DATUM:		  19.04.2012
C:			    AVR
Compiler:	  GCC
VERSION:	  0.4.3
RELEASE:	  beta
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "can.h"
#include "Globals.h"
#include "Timer.h"

int8_t iemob[15] = {
  IEMOB0, IEMOB1, IEMOB2,  IEMOB3,  IEMOB4,  IEMOB5,  IEMOB6, IEMOB7,
  IEMOB8, IEMOB9, IEMOB10, IEMOB11, IEMOB12, IEMOB13, IEMOB14 };

// extended id
int8_t extended_id;

// Message Objects zuruecksetzen, CAN-Controller aktivieren
// Parameter:
//   uint8_t baud: 0,1,2...5 (fuer 100, 125, 200, 250, 500, 1000)
//   uint8_t intmode: Ereignis, bei dem ein Interrupt ausgelst werden soll
//     - NONE  - Deaktiviert
//     - TX    - Daten gesendet
//     - RX    - Daten empfangen
//     - TXRX  - Daten gesendet und/oder empfangen
int can_init(uint8_t baudrate, uint8_t intmode, uint8_t eid)
{
  uint8_t i;

  extended_id = eid ? 1 : 0;

  // Status- und Steuerregister aller Message Objects initialisieren
  for(i = 0; i < NO_MOBS; i++)
  {
    can_get_mob(i);
    CANSTMOB = 0;
    CANCDMOB = 0;
  }

  // set baudrate
  if(can_set_baudrate(baudrate) == 0)
    return 0;

  if(can_set_interrupt(intmode) == 0)
    return 0;

  // CAN-Controller in Enabled Mode setzen
  setbit(CANGCON, ENASTB);

  // Warten bis der CAN-Controller das Enabled-Bit gesetzt hat und
  // einsatzbereit ist
  while (!getbit(CANGSTA, ENFG));

  return 1; 
}

int can_deinit(void)
{
  uint8_t i;

  for(i = 0; i < 15; i++)
    can_disable_mob(i);

  // CAN-Controller ausschalten
  clearbit(CANGCON, ENASTB);

  // warten bis wirklich ausgechaltet
  while (getbit(CANGSTA, ENFG));

  return 1;
}

void can_set_autoreply(uint8_t mob, uint8_t length, uint8_t *buf)
{
  can_get_mob(mob);
  can_set_mode(CAN_MODE_AUTO_REPLY);
  can_set_data(length, buf);
}

int can_set_baudrate(uint8_t baudrate)
{
	#if F_CPU	== 16000000
		switch(baudrate){
			case CAN_BAUDRATE_1000K:
				CANBT1		= 0x00;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_500K:
				CANBT1		= 0x02;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_250K:
				CANBT1		= 0x06;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_200K:
				CANBT1		= 0x08;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_125K:
				CANBT1		= 0x0e;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_100K:
				CANBT1		= 0x12;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_50K:
				CANBT1		= 0x0E;
				CANBT2		= 0x0c;
				CANBT3		= 0x4C;
				return 1;
				break;

			default:
				return 0;
				break;
		}
	#elif F_CPU	== 12000000
		switch(baudrate){
			case CAN_BAUDRATE_1000K:
				CANBT1		= 0x00;
				CANBT2		= 0x08;
				CANBT3		= 0x25;
				return 1;
				break;

			case CAN_BAUDRATE_500K:
				CANBT1		= 0x02;
				CANBT2		= 0x08;
				CANBT3		= 0x25;
				return 1;
				break;

			case CAN_BAUDRATE_250K:
				CANBT1		= 0x04;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_200K:
				CANBT1		= 0x04;
				CANBT2		= 0x0e;
				CANBT3		= 0x4b;
				return 1;
				break;

			case CAN_BAUDRATE_125K:
				CANBT1		= 0x0a;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_100K:
				CANBT1		= 0x0a;
				CANBT2		= 0x0e;
				CANBT3		= 0x4b;
				return 1;
				break;

			default:
				return 0;
				break;
		}
	#elif F_CPU	==  8000000
		switch(baudrate){
			case CAN_BAUDRATE_1000K:
				CANBT1		= 0x00;
				CANBT2		= 0x04;
				CANBT3		= 0x13;
				return 1;
				break;

			case CAN_BAUDRATE_500K:
				CANBT1		= 0x00;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_250K:
				CANBT1		= 0x02;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_200K:
				CANBT1		= 0x02;
				CANBT2		= 0x0e;
				CANBT3		= 0x4b;
				return 1;
				break;

			case CAN_BAUDRATE_125K:
				CANBT1		= 0x06;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_100K:
				CANBT1		= 0x08;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			default:
				return 0;
				break;
		}
	#elif F_CPU	==  6000000
		switch(baudrate){
			case CAN_BAUDRATE_1000K:
				return 0;
				break;

			case CAN_BAUDRATE_500K:
				CANBT1		= 0x00;
				CANBT2		= 0x08;
				CANBT3		= 0x25;
				return 1;
				break;

			case CAN_BAUDRATE_250K:
				CANBT1		= 0x02;
				CANBT2		= 0x08;
				CANBT3		= 0x25;
				return 1;
				break;

			case CAN_BAUDRATE_200K:
				CANBT1		= 0x02;
				CANBT2		= 0x0c;
				CANBT3		= 0x35;
				return 1;
				break;

			case CAN_BAUDRATE_125K:
				CANBT1		= 0x04;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_100K:
				CANBT1		= 0x04;
				CANBT2		= 0x0e;
				CANBT3		= 0x4b;
				return 1;
				break;

			default:
				return 0;
				break;
		}
	#elif F_CPU	==  4000000
		switch(baudrate){
			case CAN_BAUDRATE_1000K:
				return 0;
				break;

			case CAN_BAUDRATE_500K:
				CANBT1		= 0x00;
				CANBT2		= 0x04;
				CANBT3		= 0x13;
				return 1;
				break;

			case CAN_BAUDRATE_250K:
				CANBT1		= 0x02;
				CANBT2		= 0x04;
				CANBT3		= 0x13;
				return 1;
				break;

			case CAN_BAUDRATE_200K:
				CANBT1		= 0x00;
				CANBT2		= 0x0e;
				CANBT3		= 0x4b;
				return 1;
				break;

			case CAN_BAUDRATE_125K:
				CANBT1		= 0x02;
				CANBT2		= 0x0c;
				CANBT3		= 0x37;
				return 1;
				break;

			case CAN_BAUDRATE_100K:
				CANBT1		= 0x02;
				CANBT2		= 0x0e;
				CANBT3		= 0x4b;
				return 1;
				break;

			default:
				return 0;
				break;
		}
	#else
		#warning F_CPU has no correct frequency for baudrate settings
		return 0;
	#endif
}

// Parameter:
//   uint8_t mob: Nummer des zu whlenden Objekts (0-14)
//
//   uint8_t mode
//     - Betriebsart des Message Objekts:
//     - DISABLED         Deaktiviert
//     - TRANSMIT_DATA    Daten senden
//     - TRANSMIT_REMOTE  Anfrage senden
//     - RECEIVE_DATA     Empfangsmodus
//     - AUTO_REPLY       automatischer Antwortmodus
//
// Funktion setzt die Betriebsart des vorher gewhlten Objekts
int can_enable_mob(uint8_t mob, uint8_t mode, uint32_t id, uint32_t idm)
{
  // Objekt whlen
  can_get_mob(mob);

  // Interrupt fr dieses Objekt aktivieren
  // Parameter: uint8_t mob: Nummer des Objekts (0-14)
  // Funktion setzt den Interrupt fr das jeweilige Objekt
/*
void can_set_mob_interrupt(uint8_t mob)
{
   if (mob < 8) { setbit(CANIE2, iemob[mob]); }
   else         { setbit(CANIE1, iemob[mob]); }
}
*/

can_set_mob_interrupt(mob);

  // ID-Maske setzen
  can_set_id_mask(idm);

  // ID setzen
  can_set_id(id);

  // Betriebsmodus setzen
  if(can_set_mode(mode) == 0)
    return 0;

  return 1;
}


int can_send_data(uint8_t mob, uint8_t length, uint8_t * data)
{
  uint8_t mode;

  // Objekt whlen
  can_get_mob(mob);
  
  // Aktuelle Betriebsart sichern
  mode = can_get_mode();

  // Nutzdaten in Register schreiben
  can_set_data(length, data);

  // Datenbertragung starten
  can_set_mode(CAN_MODE_TRANSMIT_DATA);

  // Warten bis die Datenbertragung beendet ist (TXOK-Flag von CAN-Controller
  // gesetzt)
  Timeout(0, 1, 50);
  //while (!getbit(CANSTMOB, TXOK));      
  while(Timeout(0, 0, 50) == 0)
  {
	  if (!getbit(CANSTMOB, TXOK))
	  {
		  CAN_ERR = 1;
	  }
	  else 
	  {
      CAN_ERR = 0;
	    break;
	  }	  
  }//end while

  // TXOK-Flag von Hand lschen
  clearbit(CANSTMOB, TXOK);

  // Alte Betriebsart wiederherstellen
  can_set_mode(mode);

  return 1;
}


// Parameter: uint8_t mode: Ereignis, bei dem Interrupt ausgelst werden soll
//   - NONE  - Deaktiviert
//   - TX    - Daten gesendet
//   - RX    - Daten empfangen
//   - TXRX  - Daten gesendet und/oder empfangen
int can_set_interrupt(uint8_t mode)
{
  switch(mode)
  {
    case CAN_INTERRUPT_NONE:
      clearbit(CANGIE, ENIT);
      clearbit(CANGIE, ENRX);
      clearbit(CANGIE, ENTX);
      break;

    case CAN_INTERRUPT_TX:
      setbit(CANGIE, ENIT);
      clearbit(CANGIE, ENRX);
      setbit(CANGIE, ENTX);
      break;

    case CAN_INTERRUPT_RX:
      setbit(CANGIE, ENIT);
      setbit(CANGIE, ENRX);
      clearbit(CANGIE, ENTX);
      break;

    case CAN_INTERRUPT_TXRX:
      setbit(CANGIE, ENIT);
      setbit(CANGIE, ENRX);
      setbit(CANGIE, ENTX);
      break;

    default:
      return 0;
  }

  return 1;
}


// Funktion whlt CANPAGE des betreffenden Objekts aus und stellt Zugang zu
// Registern des Objekts her
void can_get_mob(uint8_t mob)
{
  CANPAGE = (mob << 4);
}

  
// Parameter: uint32_t idm: ID-Maske in Dezimalschreibweise
// Funktion setzt ID-Maske eines Objekts auf einen neuen Wert. In CANIDM4
// bleiben dabei die Werte der unteren 3 Bit (RTRTAG, Reserved und IDEMSK)
// erhalten.
void can_set_id_mask(uint32_t idm)
{
  //Standart identifier (11 bit)
  if(!extended_id)
  {
    CANIDM2 = (uint8_t)(idm << 5); 
    CANIDM1 = (uint8_t)(idm >> 3);
  }
  //extended identifier
  else
  {
    idm <<= 3;
    idm  |= 7;

    CANIDM4 = (int8_t) (idm);
    CANIDM3 = (int8_t) (idm>>8);
    CANIDM2 = (int8_t) (idm>>16);
    CANIDM1 = (int8_t) (idm>>24);
  }
}

// Funktion holt ID der empfangenen Nachricht
uint32_t can_get_id(void)
{
  uint32_t id = 0;

  //Standart identifier (11 bit)
  if(!extended_id)
  {
    id  = (uint8_t)  CANIDT2 >> 5;
    id |= (uint16_t) CANIDT1 << 3;
  }
  //extended identifier
  else
  {
    id |= ((uint32_t) CANIDT1 << 24);
    id |= ((uint32_t) CANIDT2 << 16);
    id |= ((uint32_t) CANIDT3 << 8);
    id |= (CANIDT4&0xF8);
    id >>= 3;
  }

  return id;
}

// Funktion setzt ID eines Objekts auf einen neuen Wert. In CANIDM4 bleiben
// dabei die Werte der unteren 3 Bit (RTRTAG, RB1TAG und RB0TAG) erhalten.
void can_set_id(uint32_t id)
{
  //Standart identifier (11 bit)
  if(!extended_id)
  {
    CANIDT2 = (uint8_t)(id << 5); 
    CANIDT1 = (uint8_t)(id >> 3);
  }
  //extended identifier
  else
  {
    id <<= 3;
    id &= 0xfffffff8;
    id |= (CANIDT4 & 0x07);

    CANIDT4 = (int8_t) (id);
    CANIDT3 = (int8_t) (id>>8);
    CANIDT2 = (int8_t) (id>>16);
    CANIDT1 = (int8_t) (id>>24);
  }
}


// Funktion setzt die Betriebsart des vorher gewhlten Objekts.
// Parameter: uint8_t mode:
//   - Betriebsart des Message Objekts:
//   - DISABLED        - Deaktiviert
//   - TRANSMIT_DATA   - Daten senden
//   - TRANSMIT_REMOTE - Anfrage senden
//   - RECEIVE_DATA    - Empfangsmodus
//   - AUTO_REPLY      - automatischer Antwortmodus
int can_set_mode(uint8_t mode)
{
  if(extended_id)
    setbit(CANCDMOB, IDE);
  else
    clearbit(CANCDMOB, IDE);

  switch(mode)
  {
    case CAN_MODE_DISABLED:
      clearbit(CANCDMOB, CONMOB0);
      clearbit(CANCDMOB, CONMOB1);
      clearbit(CANCDMOB, RPLV);
      clearbit(CANIDT4, RTRTAG);
      clearbit(CANIDM4, RTRMSK);
      break;

    case CAN_MODE_TRANSMIT_DATA:
      setbit(CANCDMOB, CONMOB0);
      clearbit(CANCDMOB, CONMOB1);
      clearbit(CANCDMOB, RPLV);
      clearbit(CANIDT4, RTRTAG);
      break;

    case CAN_MODE_TRANSMIT_REMOTE:
      clearbit(CANCDMOB, CONMOB1);
      setbit(CANCDMOB, CONMOB0);
      clearbit(CANCDMOB, RPLV);
      setbit(CANIDT4, RTRTAG);
      break;

    case CAN_MODE_RECEIVE_DATA:
      clearbit(CANCDMOB, CONMOB0);
      setbit(CANCDMOB, CONMOB1);
      clearbit(CANCDMOB, RPLV);
      clearbit(CANIDT4, RTRTAG);
      break;

    case CAN_MODE_AUTO_REPLY:
      clearbit(CANCDMOB, CONMOB0);
      setbit(CANCDMOB, CONMOB1);
      setbit(CANCDMOB, RPLV);
      setbit(CANIDT4, RTRTAG);
      break;

    default:
      return 0;
  }

  return 1;
}


// Funktion holt die Betriebsart des vorher gewaehlten Objekts
uint8_t can_get_mode(void)
{
  uint8_t mode = 0;

  if(!getbit(CANCDMOB, CONMOB1) && !getbit(CANCDMOB, CONMOB0))
  {
    mode = CAN_MODE_DISABLED;
  }
  else if(!getbit(CANCDMOB, CONMOB1) && getbit(CANCDMOB, CONMOB0) &&
    !getbit(CANIDT4, RTRTAG))
  {
    mode = CAN_MODE_TRANSMIT_DATA;
  }
  else if(!getbit(CANCDMOB, CONMOB1) && getbit(CANCDMOB, CONMOB0) &&
    getbit(CANIDT4, RTRTAG))
  {
    mode = CAN_MODE_TRANSMIT_REMOTE;
  }
  else if(getbit(CANCDMOB, CONMOB1) && !getbit(CANCDMOB, CONMOB0) &&
    !getbit(CANIDT4, RTRTAG))
  {
    mode = CAN_MODE_RECEIVE_DATA;
  }
  else if(getbit(CANCDMOB, CONMOB1) && !getbit(CANCDMOB, CONMOB0) &&
    getbit(CANCDMOB,RPLV) && getbit(CANIDT4, RTRTAG))
  {
    mode = CAN_MODE_AUTO_REPLY;
  }

  return mode;
}


// Funktion schreibt in das Objekt die zu uebermittelnden Daten
void can_set_data(uint8_t length, uint8_t * data)
{
  uint8_t i;
  uint8_t cancdmob;

  if(length > 8)
    length = 8;

  // Anzahl der Datenbytes in der Nachricht
  // scheinbar darf man das CANCDMOB register nicht beliebig oft
  // schreiben/lesen, daher speichern wir den wert dazwischen, loeschen die
  // entsprechenden bits fuer die laenge und schreiben dann usere laenge rein
  // wie dem auch sei: so funktionierts zumindest, also vorsicht beim aufraeumen ;-)
  cancdmob = CANCDMOB;
  clearbit(cancdmob, DLC0);
  clearbit(cancdmob, DLC1);
  clearbit(cancdmob, DLC2);
  clearbit(cancdmob, DLC3);
  CANCDMOB = (cancdmob | (length << DLC0));
  for(i = 0; i < length; i++)
    CANMSG = data[i];
}

void can_get_data(uint8_t *msg) //schreibt Daten in das bergebene Array
{
  uint8_t i;

  for(i = 0; i < 8; i++)
    msg[i] = CANMSG;
}


// Parameter: uint8_t mob: Nummer des Objekts (0-14)
// Funktion setzt den Interrupt fr das jeweilige Objekt
void can_set_mob_interrupt(uint8_t mob)
{
   if (mob < 8) { setbit(CANIE2, iemob[mob]); }
   else         { setbit(CANIE1, iemob[mob]); }
}


// Parameter: uint8_t mob: Nummer des Objekts (0-14)
// Funktion setzt den Interrupt fr das jeweilige Objekt
//void can_set_mob_interrupt(uint8_t mob)
//{
//  setbit(CANIE2, iemob[mob]);
//}

// Parameter: uint8_t mob: Nummer des Objekts (0-14)
// Funktion lscht den Interrupt fr das jeweilige Objekt
void can_clear_mob_interrupt(uint8_t mob)
{
	 if (mob < 8) { clearbit(CANIE2, iemob[mob]); }
   else         { clearbit(CANIE1, iemob[mob]); }
}

// Rckgabe: uint8_t mob: Nummer des Objekts
// Funktion ermittelt, welches Objekt Interrupt ausgeloest hat
uint8_t can_get_mob_interrupt(uint8_t lastmob)
{
  uint8_t  mob, n;
  uint16_t maske, filter_maske;
  maske = CANSIT2 | (uint16_t)(CANSIT1 << 8);
  
  filter_maske = 1;
  
  //Filtermaske erstellen, um das vorhergegangene MOB nicht erneut abzurufen
  for(n=0; n<lastmob; ++n)
  {
	  filter_maske <<= 1;
  }
  
  //Vorheriges ausgelstes MOB lschen
  maske &=~filter_maske; 

  // Wenn alle 32 Bit der Bitmaske 0 sind dann ist ein Fehler aufgetreten
  if(maske == 0)
  {
	  return NOMOB;
  } 
  
  // Die Bitmaske wird so lange nach rechts geschoben, bis Bit0 eine 1 hat.
  // Die Anzahl der Schiebeoperatoren gibt somit die Nummer des MOBs zurck
  for(mob=0; (maske & 0x01)==0; maske >>= 1, ++mob);

  //Prfen, ob das 

  // Kontrolle: Wenn mob grer als die Anzahl der verfgbaren
  // Message Objects ist das Ergebnis falsch
  if(mob > 14)
  {  
    return NOMOB;
  }  
  return mob;
}


// Funktion deaktiviert das gewhlte Objekt
int can_disable_mob(uint8_t mob)
{
  // Objekt whlen
  can_get_mob(mob);

  // Interrupt fr dieses Objekt aktivieren
  can_clear_mob_interrupt(mob);

  // Betriebsmodus setzen
  can_set_mode(CAN_MODE_DISABLED);

  return 1;
}


// Funktion sendet eine Anfrage (Remote Frame)
int can_send_remote(uint8_t mob)
{
  uint8_t mode;

  // Objekt whlen
  can_get_mob(mob);
  
  // Aktuelle Betriebsart sichern
  mode = can_get_mode();

  // Datenbertragung starten
  can_set_mode(CAN_MODE_TRANSMIT_REMOTE);

  // Warten bis die Datenbertragung beendet ist (TXOK-Flag von CAN-Controller
  // gesetzt)
  while (!getbit(CANSTMOB, TXOK));      

  // TXOK-Flag von Hand lschen
  clearbit(CANSTMOB, TXOK);

  // Alte Betriebsart wiederherstellen
  can_set_mode(mode);

  return 1;
}


//Anzahl der ausgelsten MOB IRQ's ermitteln
uint8_t get_mob_irqs(void)
{
	uint8_t n, mobirqs;
  uint16_t bitpos, mobirqreg;
  
	bitpos = 1;
  mobirqs = 0;
  mobirqreg = CANSIT2 | (uint16_t)(CANSIT1<<8);
 
  //Anzahl der ausgelsten MOB IRQ's ermitteln
  for (n=0; n<15; n++) 
  {
	  bitpos<<=1;
	  if ((mobirqreg & bitpos) == bitpos) mobirqs++;
  }//end Anzahl der ausgelsten MOB IRQ's ermitteln
  
  return mobirqs;
}


// Interrupt fr Empfang einer Nachricht
ISR (CANIT_vect)
{
	uint8_t save_canpage, n, i, cnt_mobirq, mob, lastmob;

	// Aktuelle CANPAGE sichern
	save_canpage = CANPAGE;

  lastmob = 0;
  
  // Index aller MOB ermitteln, welche den Interrupt ausgelst haben
  cnt_mobirq = get_mob_irqs();
 
	// MOB Daten holen und abspeichern
  for(n=0; n<=cnt_mobirq; n++)
  {
	  mob = can_get_mob_interrupt(lastmob);
	  
	  // Falls es kein gltiges MOB war, Daten holen abbrechen
	  if(mob != NOMOB)
	  {
	    // Objekt das den Interrupt ausgelst hat holen
	    can_get_mob(mob);

	    // XXX hier eintragen, was bei einem interrupt passieren soll XXX
	    // Id der Nachricht holen und Daten des MOBs aus CANMSG auslesen
	    //id = can_get_id();
	    can_get_data(receive_can);                                    //Daten aus aktuellem MOB
	    for (i = 0; i < 8; i++)
      { /* 15 Zeilen, 8 Spalten */
        can_byte_by_mob[mob][i] = receive_can[i];                   //Daten aus RX MOB in can byte schreiben
      }//end send_can fllen
	  }//end Falls es kein gltiges MOB war, Daten holen abbrechen
	  lastmob = mob;                                                  //Ausgelesenes MOB updaten
	  
	    switch (lastmob)
	    {
	    case 0:   rx_Timeout(0, 1, 500);
	      break;  
	    case 1:   rx_Timeout(1, 1, 500);
	      break;  
	    case 2:   rx_Timeout(2, 1, 500);
	      break;  
	    case 3:   rx_Timeout(3, 1, 500);
	      break;  
	    case 4:   rx_Timeout(4, 1, 500);
	      break;  
	    case 5:   rx_Timeout(5, 1, 500);
	      break;  
	    case 6:   rx_Timeout(6, 1, 500);
	      break;  
	    case 7:   rx_Timeout(7, 1, 500);
	      break;    
	    case 8:   rx_Timeout(8, 1, 500);
	      break;  
	    case 9:   rx_Timeout(9, 1, 500);
	      break;  
	    case 10:  rx_Timeout(10, 1, 500);
	      break;  
	    case 11:  rx_Timeout(11, 1, 500);
	      break;  
	    case 12:  rx_Timeout(12, 1, 500);
	      break;  
	    case 13:  rx_Timeout(13, 1, 500);
	      break;  
	    case 14:  rx_Timeout(14, 1, 500);
	      break;  
	    default:  //Do nothing
	      break;
	  }//end switch rx_can_ibyte	  	
	}//MOB Daten holen und abspeichern
	 
	// RXOK-Flag lschen
	clearbit(CANSTMOB, RXOK);

	// MOB auf Empfang und CAN 2.0B Standard setzen
	can_set_mode(CAN_MODE_RECEIVE_DATA);
	  
	// CANPAGE wiederherstellen
	CANPAGE = save_canpage;
	//IRQ_CAN_RX = 1;
}//end CAN_Interrupt handle